@@ -12,7 +12,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: gbottler.py,v 1.92 2004/05/25 10:46:24 atzm Exp $
+# $Id: gbottler.py,v 1.109 2004/06/28 06:45:47 atzm Exp $
 #
 
 import os
@@ -36,6 +36,7 @@ from textmanager  import TextManager
 from downloader   import Downloader
 from queryluid    import LUIDInfo
 from bottleparser import *
+from gvwindow     import *
 
 from common import *
 
@@ -93,19 +94,20 @@ class Bottler(BottleClient):
 
 			# Forward all messages?
 			if self.app.prefs.get('accept_all'):
-				self.app.send_local_message(channel, ifghost, script, sender, charset)
+				gtk.timeout_add(0, self.app.send_local_message, channel, ifghost, script, sender, charset)
 
 			# Forward only installed ghosts?
 			elif self.app.prefs.get('forward_listed'):
 				if ifghost in self.app.local_ghost:
-					self.app.send_local_message(channel, ifghost, script, sender, charset)
+					gtk.timeout_add(0, self.app.send_local_message, channel, ifghost, script, sender, charset)
 
 		# log message
-		self.app.log_window.update(mid, channel, ifghost, script, receive_time)
+		gtk.timeout_add(0, self.app.log_window.update, mid, channel, ifghost, script, receive_time)
 
 		# download
 		if self.app.prefs.get('autodl'):
-			self.downloader.get_files(self.downloader.find_url_from_script(script))
+			urls = self.downloader.find_url_from_script(script)
+			gtk.timeout_add(WAIT_NEXT_REQUEST, self.downloader.get_files, urls)
 
 	def handle_dialog_message(self, message):
 		self.app.monitor_clear()
@@ -114,7 +116,7 @@ class Bottler(BottleClient):
 	def handle_broadcast_information(self, type, forced):
 		BottleClient.handle_broadcast_information(self, type, forced)
 		if type in [VOTE, AGREE]:
-			self.app.log_votes(self.headers["mid"], type, int(self.headers["num"]))
+			gtk.timeout_add(0, self.app.log_votes, self.headers["mid"], type, int(self.headers["num"]))
 
 	def handle_all_users(self):
 		self.app.show_users()
@@ -127,8 +129,6 @@ class Bottler(BottleClient):
 
 class Application:
 	DEFAULT_TARGET = unicode(_("default"), "utf-8")
-	to_sjis = lambda x: x.encode("sjis")
-	sjis_to = lambda x: unicode(x, "sjis")
 
 	def __init__(self, host="localhost", port=9801):
 # === Definition of Basic Const === #
@@ -142,14 +142,14 @@ class Application:
 		accel_group = gtk.AccelGroup()
 
 # === Create Base Window === #
-		self.main_vbox = gtk.VBox(gtk.FALSE, 0)
-		self.main_vbox.show()
+		main_vbox = gtk.VBox(gtk.FALSE, 0)
+		main_vbox.show()
 
 		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
 		self.window.connect("destroy", self.quit, "WM destroy")
 		self.window.set_title(unicode(_(APP), "utf-8"))
 		self.window.add_accel_group(accel_group)
-		self.window.add(self.main_vbox)
+		self.window.add(main_vbox)
 
 # === Create Menubar === #
 		self.set_self_messages()
@@ -169,8 +169,8 @@ class Application:
 			# part, channel and ghost are not sensitive at startup
 			self.menu_list[m].set_sensitive(gtk.FALSE)			
 
-		self.menubar = self.item_factory.get_widget("<main>")
-		self.menubar.show()
+		main_menubar = self.item_factory.get_widget("<main>")
+		main_menubar.show()
 
 # === Create Main Editor === #
 		self.main_text = gtk.TextView()
@@ -189,20 +189,20 @@ class Application:
 		sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
 		sw.add(self.main_text)
 
-		self.frame = gtk.Frame()
-		self.frame.show()
-		self.frame.set_shadow_type(gtk.SHADOW_IN)
-		self.frame.add(sw)
+		text_frame = gtk.Frame()
+		text_frame.show()
+		text_frame.set_shadow_type(gtk.SHADOW_IN)
+		text_frame.add(sw)
 
 # === Create Buttons === #
 		self.tooltips = gtk.Tooltips()
 
-		checkButton = get_icon_button(gtk.STOCK_SPELL_CHECK,
-									  gtk.ICON_SIZE_MENU)
-		testButton = get_icon_button(gtk.STOCK_REFRESH,
-									 gtk.ICON_SIZE_MENU)
-		self.send_button = get_icon_button(gtk.STOCK_OK,
-										   gtk.ICON_SIZE_MENU)
+		checkButton       = get_icon_button(gtk.STOCK_SPELL_CHECK,
+											gtk.ICON_SIZE_MENU)
+		testButton        = get_icon_button(gtk.STOCK_REFRESH,
+											gtk.ICON_SIZE_MENU)
+		self.send_button  = get_icon_button(gtk.STOCK_OK,
+											gtk.ICON_SIZE_MENU)
 		self.clear_button = get_icon_button(gtk.STOCK_CLEAR,
 											gtk.ICON_SIZE_MENU)
 
@@ -247,34 +247,34 @@ class Application:
 		self.users.show()
 
 # === Bring Buttons and Labels together to HBox, Frame === #
-		self.toolbar = gtk.HBox(gtk.FALSE, 1)
-		self.toolbar.show()
+		main_toolbar = gtk.HBox(gtk.FALSE, 1)
+		main_toolbar.show()
 
-		self.toolbar.pack_start(checkButton, gtk.FALSE, gtk.TRUE, 0)
-		self.toolbar.pack_start(testButton, gtk.FALSE, gtk.TRUE, 0)
-		self.toolbar.pack_start(self.send_button, gtk.FALSE, gtk.TRUE, 0)
-		self.toolbar.pack_start(self.clear_button, gtk.FALSE, gtk.TRUE, 0)
+		main_toolbar.pack_start(checkButton, gtk.FALSE, gtk.TRUE, 0)
+		main_toolbar.pack_start(testButton, gtk.FALSE, gtk.TRUE, 0)
+		main_toolbar.pack_start(self.send_button, gtk.FALSE, gtk.TRUE, 0)
+		main_toolbar.pack_start(self.clear_button, gtk.FALSE, gtk.TRUE, 0)
 
 		vs = gtk.VSeparator()
 		vs.show()
-		self.toolbar.pack_start(vs, gtk.FALSE, gtk.TRUE, 3)
+		main_toolbar.pack_start(vs, gtk.FALSE, gtk.TRUE, 3)
 
-		self.toolbar.pack_start(channelLabel, gtk.FALSE, gtk.TRUE, 0)
-		self.toolbar.pack_start(self.c_e_box, gtk.FALSE, gtk.TRUE, 0)
-		self.toolbar.pack_start(self.g_e_box, gtk.FALSE, gtk.TRUE, 0)
+		main_toolbar.pack_start(channelLabel, gtk.FALSE, gtk.TRUE, 0)
+		main_toolbar.pack_start(self.c_e_box, gtk.FALSE, gtk.TRUE, 0)
+		main_toolbar.pack_start(self.g_e_box, gtk.FALSE, gtk.TRUE, 0)
 
 		vs = gtk.VSeparator()
 		vs.show()
-		self.toolbar.pack_start(vs, gtk.FALSE, gtk.TRUE, 3)
+		main_toolbar.pack_start(vs, gtk.FALSE, gtk.TRUE, 3)
 
-		self.toolbar.pack_start(usersLabel, gtk.FALSE, gtk.TRUE, 0)
-		self.toolbar.pack_start(self.users, gtk.FALSE, gtk.TRUE, 0)
+		main_toolbar.pack_start(usersLabel, gtk.FALSE, gtk.TRUE, 0)
+		main_toolbar.pack_start(self.users, gtk.FALSE, gtk.TRUE, 0)
 
-		self.toolframe = gtk.Frame()
-		self.toolframe.show()
-		self.toolframe.set_border_width(0)
-		self.toolframe.set_shadow_type(gtk.SHADOW_OUT)
-		self.toolframe.add(self.toolbar)
+		toolframe = gtk.Frame()
+		toolframe.show()
+		toolframe.set_border_width(0)
+		toolframe.set_shadow_type(gtk.SHADOW_OUT)
+		toolframe.add(main_toolbar)
 
 # === Create Popup Menu from Main Editor === #
 		pmsg = self.get_popup_menu_messages()
@@ -291,11 +291,11 @@ class Application:
 		self.pbar.show()
 		self.pbar.set_size_request(380, 20)
 
-		self.pframe = gtk.Frame()
-		self.pframe.show()
-		self.pframe.set_shadow_type(gtk.SHADOW_IN)
-		self.pframe.set_border_width(0)
-		self.pframe.add(self.pbar)
+		progress_frame = gtk.Frame()
+		progress_frame.show()
+		progress_frame.set_shadow_type(gtk.SHADOW_IN)
+		progress_frame.set_border_width(0)
+		progress_frame.add(self.pbar)
 
 # === Create Statusbar as window resize grip === #
 		self.sbar = gtk.Statusbar()
@@ -303,21 +303,21 @@ class Application:
 		self.sbar.set_has_resize_grip(gtk.TRUE)
 
 # === Bring Progressbar and Statusbar together to HBox === #
-		self.barbox = gtk.HBox()
-		self.barbox.show()
-		self.barbox.pack_start(self.pframe)
-		self.barbox.pack_start(self.sbar)
+		statusbar_box = gtk.HBox()
+		statusbar_box.show()
+		statusbar_box.pack_start(progress_frame)
+		statusbar_box.pack_start(self.sbar)
 
 # === Bring Main Editor and Progressbar together to VPaned === #
-		self.paned = gtk.VPaned()
-		self.paned.show()
-		self.paned.pack1(self.frame, gtk.TRUE)
-		self.paned.pack2(self.barbox, gtk.FALSE, gtk.FALSE)
+		main_paned = gtk.VPaned()
+		main_paned.show()
+		main_paned.pack1(text_frame, gtk.TRUE)
+		main_paned.pack2(statusbar_box, gtk.FALSE, gtk.FALSE)
 
 # === Bring all items together to Main Window Finally === #
-		self.main_vbox.pack_start(self.menubar, gtk.FALSE, gtk.TRUE, 0)
-		self.main_vbox.pack_start(self.toolframe, gtk.FALSE, gtk.TRUE, 0)
-		self.main_vbox.pack_start(self.paned, gtk.TRUE, gtk.TRUE, 0) # main editor and progressbar
+		main_vbox.pack_start(main_menubar, gtk.FALSE, gtk.TRUE, 0)
+		main_vbox.pack_start(toolframe, gtk.FALSE, gtk.TRUE, 0)
+		main_vbox.pack_start(main_paned, gtk.TRUE, gtk.TRUE, 0) # main editor and progressbar
 
 		self.window.show()
 		self.main_text.grab_focus()
@@ -340,16 +340,18 @@ class Application:
 			self.luid_info = LUIDInfo(luid)
 
 		self.log_window = LogWindow(self)
-		self.phrase_manager = PhraseManager()
-		self.phrase_window = PhraseWindow(self, self.phrase_manager)
+
+		self.gvwindow = GVWindow(os.path.join(open_bottlecase(), 'gviewer', 'Ghost'),
+								 os.path.join(open_bottlecase(), 'gviewer', 'ghost.txt'))
 
 		self.menu_list["/%s/%s" % (self.msg["window"][1],
 								   self.msg["logs"])].connect("activate", self.log_window.open, None)
 
 		self.menu_list["/%s/%s" % (self.msg["window"][1],
-								   self.msg["phrases"])].connect("activate", self.phrase_window.open, None)
+								   self.msg["phrases"])].connect("activate", self.phrase_open, None)
 
-		self.about = None
+		self.menu_list["/%s/%s" % (self.msg["window"][1],
+								   self.msg["gviewer"])].connect("activate", self.gvwindow.open, None)
 
 		self.execute_command = 'GetNames'
 		self.target_ghost = ""
@@ -372,6 +374,30 @@ class Application:
 		if self.prefs.get('auto_connect'):
 			self.join(None, None)
 
+		self.notify_preferences_changed(self.prefs.get_all())
+
+# === Setting Selections === #
+		for target in ["CLIPBOARD", "PRIMARY"]:
+			for type in ["UTF8_STRING", "STRING", "TEXT", "COMPOUND_TEXT"]:
+				self.window.selection_add_target(target, type, 0)
+		self.window.connect("selection-get", self.selection_handle)
+		self.clipboard = ''
+
+	def dispose_selection(self):
+		for target in ["CLIPBOARD", "PRIMARY"]:
+			self.window.selection_owner_set(target)
+
+	def selection_handle(self, widget, selection_data, info, time_stamp):
+		selection_data.set_text(self.clipboard, -1)
+
+	def copy_clipboard(self, text):
+		if not text:
+			return
+		self.clipboard = text
+
+	def phrase_open(self, widget=None, data=None):
+		PhraseWindow(self, PhraseManager()).open()
+
 	def get_initial_script(self):
 		scr = self.prefs.get('initialscript')
 		n = string.find(scr, '|')
@@ -424,6 +450,11 @@ class Application:
 		self.log_window.color_changed(prefs['script'], prefs['sakura'], prefs['kero'],
 									  prefs['sync'], prefs['error'], prefs['URL'])
 
+		# notify GVWindow
+		self.gvwindow.notify_use_gviewer(prefs['use_gviewer'])
+		self.gvwindow.notify_wait_1_char(prefs['wait_1_char'])
+		self.gvwindow.notify_wait_w1_tag(prefs['wait_w1_tag'])
+
 		# notify self attributes
 		self.textmanager.color_changed(prefs['script'], prefs['sakura'], prefs['kero'],
 									   prefs['sync'], prefs['error'], prefs['URL'])
@@ -926,6 +957,8 @@ class Application:
 		gtk.timeout_add(0, self.send, widget, data, channel, script, ghost)
 
 	def send(self, widget, data, channel, script, ghost):
+		self.sent_log(channel, script, ghost)
+
 		self.monitor_clear()
 		self.monitor_insert(unicode(_("Broadcasting..."), "utf8"))
 		try:
@@ -946,6 +979,22 @@ class Application:
 
 		self.pbar.set_fraction(0.0)
 
+	def sent_log(self, channel, script, ghost):
+		if not self.prefs.get('sent_log'):
+			return
+
+		now     = time.strftime("%Y/%m/%d-%H:%M:%S", time.localtime(time.time()))
+		channel = self.client.channels[channel]["name"]
+		line    = "%s\t%s\t%s\t%s\n" % (now, ghost, channel, script)
+
+		try:
+			file = open(os.path.join(open_bottlecase(), 'sent.log'), 'a')
+		except IOError:
+			open_error_dialog('Logging failed', self.window)
+		else:
+			file.write(line)
+			file.close()
+
 	def get_script(self):
 		[si, ei] = self.editor.get_bounds()
 		script = string.replace(self.editor.get_text(si, ei), "\n", "")
@@ -955,17 +1004,11 @@ class Application:
 		return unicode(script, "utf-8")
 
 	def about(self, data, widget):
-		if self.about:
-			return
-
-		def aboutdestroy(widget, data):
-			self.about.destroy()
-			self.about = None
-
-		self.about = gtk.Dialog(unicode(_("About"), "utf-8"), parent=self.window,
-								flags=gtk.DIALOG_NO_SEPARATOR)
-		self.about.connect("destroy", aboutdestroy, "WM destroy")
-		self.about.set_border_width(10)
+		about_dialog = gtk.Dialog(unicode(_("About"), "utf-8"), self.window,
+								  gtk.DIALOG_DESTROY_WITH_PARENT|gtk.DIALOG_MODAL|gtk.DIALOG_NO_SEPARATOR,
+								  (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
+		about_dialog.connect("destroy", lambda w: w.destroy)
+		about_dialog.set_border_width(10)
 
 		h = gtk.HBox()
 		i = gtk.Image()
@@ -976,20 +1019,21 @@ class Application:
 		l.show()
 		h.pack_start(l, gtk.FALSE, gtk.TRUE, 0)
 		h.show()
-		self.about.vbox.add(h)
+		about_dialog.vbox.add(h)
 		s = gtk.HSeparator()
 		s.show()
-		self.about.vbox.add(s)
+		about_dialog.vbox.add(s)
 
 		frame = gtk.Frame(label=" %s %s " % (APP, VER))
 		l = gtk.Label("\n"+
 					  " Copyright(C) 2001-'04, Tamito KAJIYAMA \n"+
-					  " Copyright(C) 2003-'04, Atzm WATANABE \n")
-		l.set_justify(gtk.JUSTIFY_RIGHT)
+					  " Copyright(C) 2003-'04, Atzm WATANABE \n" +
+					  " Copyright(C) 2004,     Takuya KAWAHARA \n")
+		l.set_justify(gtk.JUSTIFY_LEFT)
 		l.show()
 		frame.add(l)
 		frame.show()
-		self.about.vbox.add(frame)
+		about_dialog.vbox.add(frame)
 
 		frame = gtk.Frame(label=unicode(_(" License "), "utf-8"))
 		l = gtk.Label("\n"+
@@ -998,14 +1042,11 @@ class Application:
 		l.show()
 		frame.add(l)
 		frame.show()
-		self.about.vbox.add(frame)
+		about_dialog.vbox.add(frame)
+		about_dialog.show()
 
-		b = gtk.Button(unicode(_("OK"), "utf8"), gtk.STOCK_OK)
-		b.connect("clicked", aboutdestroy, None)
-		b.show()
-		self.about.action_area.pack_start(b, gtk.TRUE, gtk.TRUE, 0)
-
-		self.about.show()
+		about_dialog.run()
+		about_dialog.destroy()
 
 	def log_votes(self, mid, type, num):
 		self.log_window.log_votes(mid, type, num)
@@ -1022,6 +1063,10 @@ class Application:
 		result = self.client.send_sstp_message(self.SSTP_SEND % (sender, channel, ifghost, ifghost, script, charset),
 											   self.host, self.port)
 		#print result
+		if self.prefs.get('use_gviewer'):
+			if self.prefs.get('open_gviewer_when_play'):
+				self.gvwindow.open()
+			self.gvwindow.enqueue(channel, ifghost, script, sender)
 
 	def vote_message(self, mid, type):
 		if self.client.vote_message(mid, type):
@@ -1221,6 +1266,7 @@ class Application:
 					   unicode(_("Window(W)"), "utf-8")],
 			"logs":    unicode(_("Logs"), "utf-8"),
 			"phrases": unicode(_("Phrases"), "utf-8"),
+			"gviewer": unicode(_("GViewer"), "utf-8"),
 			"help":   [unicode(_("Help(_H)"), "utf-8"),
 					   unicode(_("Help(H)"), "utf-8")],
 			"about":  unicode(_("About"), "utf-8"),
@@ -1252,6 +1298,7 @@ class Application:
 			( "/%s" % self.msg["window"][0], None, None, 0, "<Branch>" ),
 			( "/%s/%s" % (self.msg["window"][1], self.msg["logs"]), "<control>L", None, 0, None ),
 			( "/%s/%s" % (self.msg["window"][1], self.msg["phrases"]), None, None, 0, None ),
+			( "/%s/%s" % (self.msg["window"][1], self.msg["gviewer"]), None, None, 0, None ),
 
 			( "/%s" % self.msg["help"][0], None, None, 0, "<LastBranch>" ),
 			( "/%s/%s" % (self.msg["help"][1], self.msg["about"]), None, self.about, 0, None ),
@@ -1259,6 +1306,7 @@ class Application:
 	
 		return menu_items
 
+
 class ChannelDialog(gtk.Dialog):
 	def __init__(self, master, channels):
 		self.selection = []
