@@ -11,7 +11,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: logs.py,v 1.60 2004/05/29 11:07:51 atzm Exp $
+# $Id: logs.py,v 1.77 2004/06/24 14:22:02 atzm Exp $
 #
 
 import os, re, time
@@ -27,6 +27,11 @@ import gtk
 from common import *
 
 class LogWindow:
+	TARGET_GHOSTNAME = 1
+	TARGET_CHANNEL   = 2
+	TARGET_SCRIPT    = 5
+	TARGET_ALL       = 6
+
 	def __init__(self, app):
 # === Definition of Basic Const === #
 		self.app    = app
@@ -70,12 +75,17 @@ class LogWindow:
 		self.rowsbar.show()
 		self.rowsbar.set_has_resize_grip(gtk.FALSE)
 
-		self.s_box = gtk.HBox()
-		self.s_box.show()
-		self.s_box.pack_start(self.s_a_b, gtk.FALSE, gtk.FALSE, 0)
-		self.s_box.pack_start(self.search_entry)
-		self.s_box.pack_start(self.rowsbar)
-		self.s_box.pack_start(self.sbar)
+		self.curr_rowsbar = gtk.Statusbar()
+		self.curr_rowsbar.show()
+		self.curr_rowsbar.set_has_resize_grip(gtk.FALSE)
+
+		search_hbox = gtk.HBox()
+		search_hbox.show()
+		search_hbox.pack_start(self.s_a_b, gtk.FALSE, gtk.FALSE, 0)
+		search_hbox.pack_start(self.search_entry)
+		search_hbox.pack_start(self.curr_rowsbar)
+		search_hbox.pack_start(self.rowsbar)
+		search_hbox.pack_start(self.sbar)
 
 # === Create Menubar === #
 		menu_items = self.get_menu_items()
@@ -84,20 +94,25 @@ class LogWindow:
 		self.item_factory = gtk.ItemFactory(gtk.MenuBar, "<main>", accel_group)
 		self.item_factory.create_items(menu_items)
 
-		self.menu_list = {}
-		for m in menu_items:
-			key = re.sub("_", "", m[0])
-			self.menu_list[key] = self.item_factory.get_item(key)
+		#self.menu_list = {}
+		#for m in menu_items:
+		#	key = re.sub("_", "", m[0])
+		#	self.menu_list[key] = self.item_factory.get_item(key)
 
-		self.menubar = self.item_factory.get_widget("<main>")
-		self.menubar.show()
+		main_menubar = self.item_factory.get_widget("<main>")
+		main_menubar.show()
 
 # === Create Popup Menu for CList === #
 		p_menu = (
-			( "/%s" % unicode(_("Copy to main editor")), "<control>c", self.edit_copy, 0, None ),
 			( "/%s" % unicode(_("Play")), "<control>p", self.play, 0, None ),
+			( "/%s" % unicode(_("Volley")), None, self.volley, 0, None ),
 			( "/%s" % unicode(_("Vote")), "<control><alt>v", self.voting, 0, None ),
 			( "/%s" % unicode(_("Agree")), "<control><alt>a", self.agreeing, 0, None ),
+			( '/sep', None, None, 0, "<Separator>" ),
+			( "/%s" % self.msg["copy script"], "<control>c", self.edit_copy_script, 0, None ),
+			( "/%s" % self.msg["copy ghostname"], None, self.edit_copy_ghostname, 0, None ),
+			( "/%s" % self.msg["copy channel"], None, self.edit_copy_channel, 0, None ),
+			( "/%s" % self.msg["copy all"], None, self.edit_copy_all, 0, None ),
 			)
 		self.p_ifact = gtk.ItemFactory(gtk.Menu, "<main>", accel_group)
 		self.p_ifact.create_items(p_menu)
@@ -108,42 +123,41 @@ class LogWindow:
 		self.ai = self.p_ifact.get_item("/%s" % unicode(_("Agree"), "utf-8"))
 
 # === Create Log Viewer === #
-		self.main_text = gtk.TextView()
-		self.main_text.show()
-		self.main_text.set_cursor_visible(gtk.FALSE)
-		self.main_text.set_editable(gtk.FALSE)
-		self.main_text.set_wrap_mode(gtk.WRAP_CHAR)
-		self.main_text.set_border_width(0)
-		self.main_text.connect("focus-in-event", self.focus_active_loglist)
+		main_text = gtk.TextView()
+		main_text.show()
+		main_text.set_cursor_visible(gtk.FALSE)
+		main_text.set_editable(gtk.FALSE)
+		main_text.set_wrap_mode(gtk.WRAP_CHAR)
+		main_text.set_border_width(0)
+		main_text.connect("focus-in-event", self.focus_active_loglist)
 
 		self.sw2 = gtk.ScrolledWindow()
 		self.sw2.show()
 		self.sw2.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
 		self.sw2.set_border_width(0)
-		self.sw2.add(self.main_text)
+		self.sw2.add(main_text)
 
-		self.frame = gtk.Frame()
-		self.frame.show()
-		self.frame.set_shadow_type(gtk.SHADOW_IN)
-		self.frame.set_border_width(0)
-		self.frame.add(self.sw2)
+		text_frame = gtk.Frame()
+		text_frame.show()
+		text_frame.set_shadow_type(gtk.SHADOW_IN)
+		text_frame.set_border_width(0)
+		text_frame.add(self.sw2)
 
-		self.monitor = self.main_text.get_buffer()
-		self.textmanager = TextManager(self, self.main_text, self.app.prefs,
+		self.textmanager = TextManager(self, main_text, self.app.prefs,
 									   BottleParser('loose'))
 
 # === Create Buttons === #
-		self.fb = get_icon_button(gtk.STOCK_CONVERT, gtk.ICON_SIZE_MENU)
-		self.ob = get_icon_button(gtk.STOCK_OPEN,    gtk.ICON_SIZE_MENU)
-		self.sb = get_icon_button(gtk.STOCK_SAVE_AS, gtk.ICON_SIZE_MENU)
+		fetch_button = get_icon_button(gtk.STOCK_CONVERT, gtk.ICON_SIZE_MENU)
+		open_button  = get_icon_button(gtk.STOCK_OPEN,    gtk.ICON_SIZE_MENU)
+		save_button  = get_icon_button(gtk.STOCK_SAVE_AS, gtk.ICON_SIZE_MENU)
 
 		self.pb = get_icon_button(gtk.STOCK_REFRESH, gtk.ICON_SIZE_MENU)
 		self.vb = get_icon_button(gtk.STOCK_YES,     gtk.ICON_SIZE_MENU)
 		self.ab = get_icon_button(gtk.STOCK_NO,      gtk.ICON_SIZE_MENU)
 
-		self.fb.connect("clicked", self.fetch)
-		self.ob.connect("clicked", self.open_xml_log)
-		self.sb.connect("clicked", self.save_as_xml)
+		fetch_button.connect("clicked", self.fetch)
+		open_button.connect("clicked", self.open_xml_log)
+		save_button.connect("clicked", self.save_as_xml)
 
 		self.pb.connect("clicked", self.play, None)
 		self.vb.connect("clicked", self.voting, None)
@@ -152,25 +166,25 @@ class LogWindow:
 		self.vb.set_sensitive(gtk.FALSE)
 		self.ab.set_sensitive(gtk.FALSE)
 
-		self.fb.connect("focus-in-event", self.focus_active_loglist)
-		self.ob.connect("focus-in-event", self.focus_active_loglist)
-		self.sb.connect("focus-in-event", self.focus_active_loglist)
+		fetch_button.connect("focus-in-event", self.focus_active_loglist)
+		open_button.connect("focus-in-event", self.focus_active_loglist)
+		save_button.connect("focus-in-event", self.focus_active_loglist)
 		self.pb.connect("focus-in-event", self.focus_active_loglist)
 		self.vb.connect("focus-in-event", self.focus_active_loglist)
 		self.ab.connect("focus-in-event", self.focus_active_loglist)
 
 		self.tooltips = gtk.Tooltips()
-		self.tooltips.set_tip(self.fb, self.msg['fetch'])
-		self.tooltips.set_tip(self.ob, self.msg['open_xml'])
-		self.tooltips.set_tip(self.sb, self.msg['save'])
+		self.tooltips.set_tip(fetch_button, self.msg['fetch'])
+		self.tooltips.set_tip(open_button, self.msg['open_xml'])
+		self.tooltips.set_tip(save_button, self.msg['save'])
 
 		self.tooltips.set_tip(self.pb, unicode(_("Play"), "utf-8"))
 		self.tooltips.set_tip(self.vb, unicode(_("Vote"), "utf-8"))
 		self.tooltips.set_tip(self.ab, unicode(_("Agree"), "utf-8"))
 
 # === Create Labels and style === #
-		self.m = gtk.Label(unicode(_("Style"), "utf-8") + ": ")
-		self.m.show()
+		style_desc = gtk.Label(unicode(_("Style"), "utf-8") + ": ")
+		style_desc.show()
 
 		self.style_l = gtk.Label(self.msg['talk style'])
 		self.style_l.show()
@@ -184,31 +198,31 @@ class LogWindow:
 																	  self.msg["style"][1])))
 
 # === Bring Buttons and Labels together to HBox, Frame === #
-		self.hbox = gtk.HBox()
-		self.hbox.show()
+		toolbox = gtk.HBox()
+		toolbox.show()
 
-		self.hbox.pack_start(self.fb, gtk.FALSE, gtk.TRUE, 0)
-		self.hbox.pack_start(self.ob, gtk.FALSE, gtk.TRUE, 0)
-		self.hbox.pack_start(self.sb, gtk.FALSE, gtk.TRUE, 0)
+		toolbox.pack_start(fetch_button, gtk.FALSE, gtk.TRUE, 0)
+		toolbox.pack_start(open_button, gtk.FALSE, gtk.TRUE, 0)
+		toolbox.pack_start(save_button, gtk.FALSE, gtk.TRUE, 0)
 		vs = gtk.VSeparator()
 		vs.show()
-		self.hbox.pack_start(vs, gtk.FALSE, gtk.TRUE, 3)
+		toolbox.pack_start(vs, gtk.FALSE, gtk.TRUE, 3)
 
-		self.hbox.pack_start(self.pb, gtk.FALSE, gtk.TRUE, 0)
-		self.hbox.pack_start(self.vb, gtk.FALSE, gtk.TRUE, 0)
-		self.hbox.pack_start(self.ab, gtk.FALSE, gtk.TRUE, 0)
+		toolbox.pack_start(self.pb, gtk.FALSE, gtk.TRUE, 0)
+		toolbox.pack_start(self.vb, gtk.FALSE, gtk.TRUE, 0)
+		toolbox.pack_start(self.ab, gtk.FALSE, gtk.TRUE, 0)
 		vs = gtk.VSeparator()
 		vs.show()
-		self.hbox.pack_start(vs, gtk.FALSE, gtk.TRUE, 3)
+		toolbox.pack_start(vs, gtk.FALSE, gtk.TRUE, 3)
 
-		self.hbox.pack_start(self.m, gtk.FALSE, gtk.TRUE, 0)
-		self.hbox.pack_start(self.s_e_box, gtk.FALSE, gtk.TRUE, 0)
+		toolbox.pack_start(style_desc, gtk.FALSE, gtk.TRUE, 0)
+		toolbox.pack_start(self.s_e_box, gtk.FALSE, gtk.TRUE, 0)
 
-		self.bframe = gtk.Frame()
-		self.bframe.show()
-		self.bframe.set_border_width(0)
-		self.bframe.set_shadow_type(gtk.SHADOW_OUT)
-		self.bframe.add(self.hbox)
+		button_frame = gtk.Frame()
+		button_frame.show()
+		button_frame.set_border_width(0)
+		button_frame.set_shadow_type(gtk.SHADOW_OUT)
+		button_frame.add(toolbox)
 
 # === Create Notebook === #
 		label = unicode(_('Current'), 'utf-8')
@@ -219,26 +233,25 @@ class LogWindow:
 		self.create_tab(label)
 
 # === Bring Log Entry and Log Viewer together to VPaned === #
-		self.pane = gtk.VPaned()
-		self.pane.show()
-		self.pane.pack1(self.logbook, gtk.TRUE)
-		self.pane.pack2(self.frame, gtk.FALSE)
+		main_paned = gtk.VPaned()
+		main_paned.show()
+		main_paned.pack1(self.logbook, gtk.TRUE)
+		main_paned.pack2(text_frame, gtk.FALSE)
 
 # === Bring all items together to Main Window Finally === #
-		self.main_vbox = gtk.VBox(gtk.FALSE, 0)
-		self.main_vbox.show()
+		main_vbox = gtk.VBox(gtk.FALSE, 0)
+		main_vbox.show()
 
-		self.main_vbox.pack_start(self.menubar, gtk.FALSE, gtk.TRUE, 0)
-		self.main_vbox.pack_start(self.bframe, gtk.FALSE, gtk.TRUE, 0)
-		self.main_vbox.pack_start(self.pane)
-		self.main_vbox.pack_start(self.s_box, gtk.FALSE, gtk.FALSE, 0)
+		main_vbox.pack_start(main_menubar, gtk.FALSE, gtk.TRUE, 0)
+		main_vbox.pack_start(button_frame, gtk.FALSE, gtk.TRUE, 0)
+		main_vbox.pack_start(main_paned)
+		main_vbox.pack_start(search_hbox, gtk.FALSE, gtk.FALSE, 0)
 
-		self.window.add(self.main_vbox)
+		self.window.add(main_vbox)
 
 # === Initialization of LogWindow === #
 		self.focus_active_loglist()
 
-# === Open and Close === #
 	def open(self, widget, data):
 		self.window.show()
 
@@ -346,13 +359,23 @@ class LogWindow:
 		filew.show()
 
 	def fetch(self, widget=None, data=None):
-		d = LogFetchDialog(self.window)
+		try:
+			channels = self.app.client.channels
+		except:
+			channels = []
+
+		d = LogFetchDialog(self.window, channels)
 		if not d.request or not d.request_name:
 			return
 
 		fetcher = LogFetcher()
 		logs    = fetcher.fetch(d.request)
 
+		if type(logs) is StringType:
+			open_error_dialog(unicode(_('Server returns some errors'), 'utf-8') + ':\n   ' + \
+							  logs, self.window)
+			return
+
  		self.create_tab(d.request_name, list=logs,
 						focus=self.app.prefs.get('logtabfocus'),
 						pos=self.app.prefs.get('logpagepos'))
@@ -374,6 +397,18 @@ class LogWindow:
 		self.logbook.reorder_child(sender, receiver_n)
 		self.logbook.reorder_child(receiver, sender_n)
 
+	def edit_copy_ghostname(self, widget, data):
+		self.edit_copy(widget, data, self.TARGET_GHOSTNAME)
+	def edit_copy_channel(self, widget, data):
+		self.edit_copy(widget, data, self.TARGET_CHANNEL)
+	def edit_copy_script(self, widget, data):
+		self.edit_copy(widget, data, self.TARGET_SCRIPT)
+	def edit_copy_all(self, widget, data):
+		self.edit_copy(widget, data, self.TARGET_ALL)
+	def edit_copy(self, widget, data, target):
+		self.app.dispose_selection()
+		self.app.copy_clipboard(self.logbook.edit_copy(widget, data, target))
+
 	def connect_for_logtab(self, list, child, eventbox):
 		save_menuitem = gtk.MenuItem(self.msg["save"])
 		save_menuitem.show()
@@ -404,7 +439,7 @@ class LogWindow:
 
 		list = child.get_child()
 		if self.logbook.get_n_pages() == 1:
-			self.update_rows(list.rows)
+			self.update_rows(list)
 			if list.rows > 0 and list.selection:
 				self.select(list, list.selection[0])
 		self.connect_list(list)
@@ -435,12 +470,12 @@ class LogWindow:
 				if list == self.logbook.get_active_loglist():
 					if list.selection:
 						self.select(list, list.selection[0])
-					self.update_rows(list.rows)
+					self.update_rows(list)
 			self.focus_active_loglist()
 			self.connect_for_logtab(list, child, eventbox)
 
 		elif self.logbook.current_is(self.logbook.get_active_loglist()):
-			self.update_rows(self.logbook.get_active_loglist().rows)
+			self.update_rows(self.logbook.get_active_loglist())
 
 	def close_current_tab(self, widget, child):
 		self.app.notify_log_current_tab_closed()
@@ -456,10 +491,6 @@ class LogWindow:
 		list.connect("key-press-event",    self.loglist_keypress)
 		list.connect("button-press-event", self.popup)
 
-	def update_rows(self, rows):
-		self.rowsbar.pop(0)
-		self.rowsbar.push(0, str(rows) + unicode(_('Messages'), 'utf-8'))
-
 	def remove_status(self):
 		self.sbar.pop(0)
 		self.rowsbar.pop(0)
@@ -471,7 +502,7 @@ class LogWindow:
 			self.remove_status()
 		else:
 			self.select(list, list.selection[0])
-			self.update_rows(list.rows)
+			self.update_rows(list)
 
 	def tab_pos_changed(self, pos):
 		if self.logbook.get_tab_pos() != pos:
@@ -482,27 +513,36 @@ class LogWindow:
 							self.search_entry.get_text(),
 							widget, data)
 
-	def play(self, widget=None, data=None):
+	def volley(self, widget=None, data=None):
 		list = self.logbook.get_active_loglist()
 		if not list or not list.selection:
 			return
-		script  = unicode(list.get_text(list.selection[0], 5), 'utf-8')
-		ifghost = unicode(list.get_text(list.selection[0], 1), 'utf-8')
-		channel = unicode(list.get_text(list.selection[0], 2), 'utf-8')
+
+		gtk.timeout_add(0, self._volley, widget, data, list, list.selection[0], 0)
+
+	def _volley(self, widget, data, list, target, min):
+		gtk.timeout_add(0, self.play, widget, data, list, target)
+		if target > min:
+			gtk.timeout_add(WAIT_NEXT_REQUEST, self._volley, widget, data, list, target-1, min)
+
+	def play(self, widget=None, data=None, list=None, target=None):
+		if list is None or target is None:
+			list = self.logbook.get_active_loglist()
+			if not list or not list.selection:
+				return
+			target = list.selection[0]
+
+		script  = unicode(list.get_text(target, 5), 'utf-8')
+		ifghost = unicode(list.get_text(target, 1), 'utf-8')
+		channel = unicode(list.get_text(target, 2), 'utf-8')
+
 		if script is None or ifghost is None:
 			return
-		self.app.send_local_message(channel, ifghost, script)
+		gtk.timeout_add(WAIT_NEXT_REQUEST, self.app.send_local_message, channel, ifghost, script)
 
 	def log_votes(self, mid, type, num):
 		self.logbook.log_votes(mid, type, num)
 
-	def edit_copy(self, widget, data):
-		script = self.logbook.edit_copy(widget, data)
-		if not script:
-			return
-		script = re.sub(r'\\n', r'\\n\n', script)
-		self.app.insert(script)
-
 	def voting(self, widget, data):
 		mid, type = self.logbook.voting(widget, data, self.window)
 		if mid is None or type is None:
@@ -536,6 +576,17 @@ class LogWindow:
 			elif event.type == 5:
 				self.play(loglist, event)
 
+	def update_rows(self, list):
+		if list and list.rows:
+			self.rowsbar.pop(0)
+			self.rowsbar.push(0, str(list.rows) + unicode(_('Messages'), 'utf-8'))
+		self.update_select_no(list)
+
+	def update_select_no(self, list):
+		if list and list.selection and len(list.selection) > 0:
+			self.curr_rowsbar.pop(0)
+			self.curr_rowsbar.push(0, unicode('No.' + str(list.selection[0]+1), 'utf-8'))
+
 	def select(self, loglist, row, col=None, event=None):
 		self.textmanager.clear()
 		script = unicode(loglist.get_text(row, 5), "utf-8", 'replace')
@@ -543,6 +594,7 @@ class LogWindow:
 		self.sw2.emit('scroll-child', gtk.SCROLL_START, gtk.FALSE)
 		self.sbar.pop(0)
 		self.sbar.push(0, str(len(script.encode('sjis'))) + unicode(_('bytes'), 'utf-8'))
+		self.update_select_no(loglist)
 
 	def scroll(self, widget, event):
 		if event.state == 1: # shift
@@ -594,11 +646,18 @@ class LogWindow:
 			( "/%s/%s" % (self.app.msg["file"][1], self.msg["close"]), "<control>q", self.close, 0, None ),
 
 			( "/%s" % self.app.msg["edit"][0], None, None, 0, "<Branch>" ),
-			( "/%s/%s" % (self.app.msg["edit"][1], self.msg["copy to main editor"]),
-			  "<control>c", self.edit_copy, 0, None ),
+			( "/%s/%s" % (self.app.msg["edit"][1], self.msg["copy script"]),
+			  "<control>c", self.edit_copy_script, 0, None ),
+			( "/%s/%s" % (self.app.msg["edit"][1], self.msg["copy ghostname"]),
+			  None, self.edit_copy_ghostname, 0, None ),
+			( "/%s/%s" % (self.app.msg["edit"][1], self.msg["copy channel"]),
+			  None, self.edit_copy_channel, 0, None ),
+			( "/%s/%s" % (self.app.msg["edit"][1], self.msg["copy all"]),
+			  None, self.edit_copy_all, 0, None ),
+			( "/%s/sep1" % self.app.msg["edit"][1], None, None, 0, "<Separator>" ),
 			( "/%s/%s" % (self.app.msg["edit"][1], self.msg["search"]), "<control>f",
 			  lambda x, y: self.search_entry.grab_focus(), 0, None ),
-			( "/%s/sep" % self.app.msg["edit"][1], None, None, 0, "<Separator>" ),
+			( "/%s/sep2" % self.app.msg["edit"][1], None, None, 0, "<Separator>" ),
 			( "/%s/%s" % (self.app.msg["edit"][1], self.app.msg["pref"][0]), None,
 			  self.app.open_preference, 0, None ),
 
@@ -630,7 +689,10 @@ class LogWindow:
 			"save":  unicode(_("Save as XML"), "utf-8"),
 			"fetch":  unicode(_("Fetch logs"), "utf-8"),
 			"close":  unicode(_("Close"), "utf-8"),
-			"copy to main editor":   unicode(_("Copy to main editor"), "utf-8"),
+			"copy script":   unicode(_("Copy script"), "utf-8"),
+			"copy ghostname": unicode(_("Copy ghost name"), "utf-8"),
+			"copy channel":   unicode(_("Copy channel"), "utf-8"),
+			"copy all":   unicode(_("Copy all"), "utf-8"),
 			"search":   unicode(_("Search"), "utf-8"),
 
 			"view":		   [unicode(_("View(_V)"), "utf-8"),
@@ -797,14 +859,19 @@ class LogBook(gtk.Notebook):
 			return None, None
 		return list.agreeing(widget, data, parent)
 
-	def edit_copy(self, widget, data):
+	def edit_copy(self, widget, data, target):
 		list = self.get_active_loglist()
 		if not list or not list.selection:
 			return
-		script = list.get_text(list.selection[0], 5)
-		if not script:
+		text = ''
+		if target == LogWindow.TARGET_ALL:
+			for i in range(list.columns):
+				text += ' ' + list.get_text(list.selection[0], i)
+		else:
+			text = list.get_text(list.selection[0], target)
+		if not text:
 			return
-		return script
+		return text
 
 
 class LogList(gtk.CList):
