@@ -10,7 +10,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: prefs.py,v 1.42 2004/05/28 16:13:23 atzm Exp $
+# $Id: prefs.py,v 1.47 2004/06/17 15:58:59 atzm Exp $
 #
 
 import os, sys, re, string
@@ -27,13 +27,14 @@ class PreferenceDialog:
 
 # === Create General Tab === #
 		self.prefs['auto_open_log'] = gtk.CheckButton(unicode(_("Open log window when startup")))
-		self.prefs['auto_connect'] = gtk.CheckButton(unicode(_("Connect bottle server when startup")))
-		self.prefs['auto_join'] = gtk.CheckButton(unicode(_("Join all channels when connect")))
-		self.prefs['auto_clear']  = gtk.CheckButton(unicode(_("Clear editor after send")))
+		self.prefs['auto_connect']  = gtk.CheckButton(unicode(_("Connect bottle server when startup")))
+		self.prefs['auto_join']     = gtk.CheckButton(unicode(_("Join all channels when connect")))
+		self.prefs['auto_clear']    = gtk.CheckButton(unicode(_("Clear editor after send")))
+		self.prefs['sent_log']      = gtk.CheckButton(unicode(_("Logging history of sent script")))
 
 		self.genbox = gtk.VBox()
 		self.genbox.show()
-		for name in ['auto_open_log', 'auto_connect', 'auto_join', 'auto_clear']:
+		for name in ['auto_open_log', 'auto_connect', 'auto_join', 'auto_clear', 'sent_log']:
 			self.prefs[name].set_border_width(5)
 			self.prefs[name].show()
 			self.genbox.pack_start(self.prefs[name], gtk.FALSE, gtk.FALSE, 0)
@@ -86,6 +87,57 @@ class PreferenceDialog:
 		self.gentab.pack_start(self.sendframe, gtk.FALSE, gtk.FALSE, 0)
 		self.gentab.pack_start(self.iniscframe, gtk.FALSE, gtk.FALSE, 0)
 
+# === Create GViewer Tab == #
+		self.prefs['use_gviewer'] = gtk.CheckButton(unicode(_('Use GViewer'), 'utf-8'))
+		self.prefs['use_gviewer'].set_border_width(2)
+		self.prefs['use_gviewer'].connect('toggled', self.viewer_toggled)
+		self.prefs['use_gviewer'].show()
+
+		self.prefs['open_gviewer_when_play'] = gtk.CheckButton(unicode(_('Auto open GViewer when play script'), 'utf-8'))
+		self.prefs['open_gviewer_when_play'].set_border_width(2)
+		self.prefs['open_gviewer_when_play'].show()
+
+		adj = gtk.Adjustment(1.0, 0.0, 500.0, 1.0, 5.0, 0.0)
+		self.prefs['wait_1_char'] = gtk.SpinButton(adj, 0, 0)
+		self.prefs['wait_1_char'].show()
+		c1_label = gtk.Label(unicode(_('Wait for a character'), 'utf-8'))
+		c1_label.show()
+		ms_label1 = gtk.Label(unicode(_('milli seconds'), 'utf-8'))
+		ms_label1.show()
+
+		adj = gtk.Adjustment(1.0, 0.0, 500.0, 1.0, 5.0, 0.0)
+		self.prefs['wait_w1_tag'] = gtk.SpinButton(adj, 0, 0)
+		self.prefs['wait_w1_tag'].show()
+		w1_label = gtk.Label(unicode(_('Wait for \\w1'), 'utf-8'))
+		w1_label.show()
+		ms_label2 = gtk.Label(unicode(_('milli seconds'), 'utf-8'))
+		ms_label2.show()
+
+		wait_table = gtk.Table(2, 3)
+		wait_table.show()
+		wait_table.attach(c1_label,                  0, 1, 0, 1, gtk.FILL, gtk.FILL, 3, 3)
+		wait_table.attach(self.prefs['wait_1_char'], 1, 2, 0, 1, gtk.FILL, gtk.FILL, 3, 3)
+		wait_table.attach(ms_label1,                 2, 3, 0, 1, gtk.FILL, gtk.FILL, 3, 3)
+		wait_table.attach(w1_label,                  0, 1, 1, 2, gtk.FILL, gtk.FILL, 3, 3)
+		wait_table.attach(self.prefs['wait_w1_tag'], 1, 2, 1, 2, gtk.FILL, gtk.FILL, 3, 3)
+		wait_table.attach(ms_label2,                 2, 3, 1, 2, gtk.FILL, gtk.FILL, 3, 3)
+
+		viewbox = gtk.VBox()
+		viewbox.show()
+		viewbox.pack_start(self.prefs['use_gviewer'], gtk.FALSE, gtk.FALSE, 0)
+		viewbox.pack_start(self.prefs['open_gviewer_when_play'], gtk.FALSE, gtk.FALSE, 0)
+		viewbox.pack_start(wait_table, gtk.FALSE, gtk.FALSE, 0)
+
+		self.viewerframe = gtk.Frame(unicode(_('GViewer'), 'utf-8'))
+		self.viewerframe.show()
+		self.viewerframe.set_border_width(5)
+		self.viewerframe.add(viewbox)
+
+		self.viewertab = gtk.VBox()
+		self.viewertab.show()
+		self.viewertab.pack_start(self.viewerframe, gtk.FALSE, gtk.FALSE, 0)
+		self.viewer_toggled()
+
 # === Create Logging Frame == #
 		self.prefs['logging'] = gtk.CheckButton(unicode(_("Save `Current' log as XML when disconnect"), 'utf-8'))
 		self.prefs['logging'].set_border_width(2)
@@ -371,6 +423,7 @@ class PreferenceDialog:
 
 		self.main_notebook.append_page(self.gentab,     gtk.Label(unicode(_("General"))))
 		self.main_notebook.append_page(self.logtab,     gtk.Label(unicode(_("Logs"))))
+		self.main_notebook.append_page(self.viewertab,  gtk.Label(unicode(_("GViewer"))))
 		self.main_notebook.append_page(self.dlframe,    gtk.Label(unicode(_("Download"))))
 		self.main_notebook.append_page(self.colorframe, gtk.Label(unicode(_("Colors"))))
 		if luid_info is not None:
@@ -450,10 +503,14 @@ class PreferenceDialog:
 		# CheckButtons
 		for name in ['auto_open_log', 'auto_connect', 'auto_join', 'autodl',
 					 'forward', 'accept_all', 'forward_listed', 'auto_clear',
-					 'logging', 'logging_current_tab_closed',
-					 'logtabfocus', 'download_log'] + DL_SUFFIX_PATTERNS.keys():
+					 'logging', 'logging_current_tab_closed', 'open_gviewer_when_play',
+					 'logtabfocus', 'download_log', 'sent_log', 'use_gviewer'] + DL_SUFFIX_PATTERNS.keys():
 			prefs[name] = self.prefs[name].get_active()
 
+		# SpinButtons
+		for name in ['wait_w1_tag', 'wait_1_char']:
+			prefs[name] = self.prefs[name].get_value_as_int()
+
 		# Entries
 		for name in ['browser', 'script', 'sakura',
 					 'kero', 'sync', 'error', 'URL',
@@ -482,6 +539,11 @@ class PreferenceDialog:
 						   'kero', 'sync', 'error', 'URL',
 						   'initialscript', 'logging_path']:
 					self.prefs[key].set_text(self.prefmanager.rc[key])
+
+				# SpinButtons
+				elif key in ['wait_w1_tag', 'wait_1_char']:
+					self.prefs[key].set_value(int(self.prefmanager.rc[key]))
+
 				else:
 					self.prefs[key].set_active(self.prefmanager.rc[key])
 
@@ -522,6 +584,12 @@ class PreferenceDialog:
 		else:
 			self.prefs['logging_path'].set_sensitive(gtk.TRUE)
 
+	def viewer_toggled(self, widget=None, data=None):
+		viewer_state = self.prefs['use_gviewer'].get_active()
+		self.prefs['open_gviewer_when_play'].set_sensitive(viewer_state)
+		self.prefs['wait_1_char'].set_sensitive(viewer_state)
+		self.prefs['wait_w1_tag'].set_sensitive(viewer_state)
+
 
 class PreferenceManager:
 	DBFILE = "bottlerc"
@@ -542,7 +610,12 @@ class PreferenceManager:
 					 ['accept_all', True],
 					 ['forward_listed', False],
 					 ['auto_clear', True],
+					 ['sent_log', True],
+					 ['use_gviewer', False],
+					 ['open_gviewer_when_play', False],
 					 ['initialscript', r'\t\u\s[10]\h\s[0]|\e'],
+					 ['wait_1_char', 47],
+					 ['wait_w1_tag', 50],
 					 ['logging', False],
 					 ['logging_current_tab_closed', False],
 					 ['logging_path', os.path.normpath(open_bottlecase() + '/autolog/%Y%m%d/%H-%M-%S.xml')],
@@ -602,3 +675,6 @@ class PreferenceManager:
 		if not self.rc.has_key(name):
 			raise ValueError("rc has no key %s\n" % name)
 		return self.rc[name]
+
+	def get_all(self):
+		return self.rc
